"""
Embedded Python Blocks:

Each time this file is saved, GRC will instantiate the first class it finds
to get ports and parameters of your block. The arguments to __init__  will
be the parameters. All of them are required to have default values!
"""

import numpy as np
from gnuradio import gr
import pmt
import threading as th

class ConvDecoderDelayTimer(gr.sync_block):
    """Conv Decoder delay timer"""

    def __init__(self, timeout=1.0):  # only default arguments here
        """arguments to this function show up as parameters in GRC"""
        gr.sync_block.__init__(
            self,
            name='Conv Decoder delay timer',   # will show up in GRC
            in_sig=[],
            out_sig=[]
        )
        # if an attribute with the same name as a parameter is found,
        # a callback is registered (properties work, too).
        self.timeout = timeout
        self.locked = th.Event()
        self.togglethread = None
        self.inport = pmt.intern('in')
        self.outport = pmt.intern('out')
        self.message_port_register_out(self.outport)
        self.message_port_register_in(self.inport)
        self.set_msg_handler(self.inport, self.lockmsg_handler)
        self.messages = [pmt.to_pmt({'dly': (0,)}), pmt.to_pmt({'dly': (1,)})]
        self.msg_index = 0

    def start(self):
        self.locked.clear()
        self.start_toggler()
        return True

    def stop(self):
        self.locked.set()   # exits thread
        self.togglethread.join()
        self.togglethread = None
        return False

    def start_toggler(self):
        if self.togglethread and self.togglethread.is_alive():
            return
        self.togglethread = th.Thread(target=self.toggle_delay)
        #self.togglethread.daemon = True
        self.togglethread.start()

    def toggle_delay(self):
        timeout = self.timeout
        locked = self.locked
        while not locked.wait(timeout):
            self.message_port_pub(self.outport, self.messages[self.msg_index])
            self.msg_index = (self.msg_index + 1) % len(self.messages)

    def lockmsg_handler(self, msg):
        if not pmt.is_pair(msg):
            return
        if pmt.eq(pmt.car(msg), pmt.intern('lock')):
            if pmt.is_true(pmt.cdr(msg)):
                self.locked.set()
            else:
                self.locked.clear()
                self.start_toggler()

    def work(self, input_items, output_items):
        """example: multiply with constant"""
        output_items[0][:] = input_items[0] * self.example_param
        return len(output_items[0])

