#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright 2016 The Aerospace Corporation.
#
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
#
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
#

from gnuradio import gr
import pmt

class CallbackMessageSink(gr.sync_block):
    """
    Executes a function callback upon receipt of a message.
    """
    def __init__(self, fcn, *args, **kwds):
        gr.sync_block.__init__(self, "callback_message_sink", None, None)
        self.fcn = fcn          # TODO: check if this is callable
        self.args = args
        self.kwds = kwds
        port = pmt.intern('msg')
        self.message_port_register_in(port)
        self.set_msg_handler(port, self.handler)

    def handler(self, msg):
        self.fcn(msg, *self.args, **self.kwds)

callback_message_sink = CallbackMessageSink
