from AbstractHeader import AbstractHeader

##
# @brief Annotation Header class.
# @details Contains the filename as well as optional annotations
# @author Miguel Dominguez
# @date 2016
class AnnotationHeader(AbstractHeader):

   ##
   # @fn headerIndex
   # @brief Return the header index of an Annotation Header.
   # @param[out] output Always 4
   @staticmethod
   def headerIndex():
      return 4

   ##
   # @fn getHeader
   # @brief Extract header data from input bytes.
   # @details This function extracts header data from a buffer and stores it
   # within the object.  A version of the data buffer with the header bytes
   # chomped off is returned
   # @param[in] Raw unsigned bytes that contain the header at the beginning
   # @param[out] The input buffer with header bytes removed
   def getHeader(self,data):
      self.headerRecordLength = (data[1] << self.BYTE_SHIFT) | data[2]
      self.annotation = "".join(map(chr,data[3:self.headerRecordLength]))
      return data[self.headerRecordLength:len(data)]

   ##
   # @fn __str__
   # @brief Returns formatted header information as a string
   # @param[out] str Header information as a string
   def __str__(self):
      output = "Annotation Header\n"
      output += "Header Record Length: {0}\n".format(self.headerRecordLength)
      output += "Annotation: {0}\n".format(self.annotation)
      return output
