from AbstractHeader import AbstractHeader

##
# @brief Image Navigation header class.
# @details Information to map the image onto earth
# @author Miguel Dominguez
# @date 2016
class ImageNavigationHeader(AbstractHeader):

   PROJECTION_NAME_LENGTH = 32
   
   ##
   # @fn headerIndex
   # @brief Return the header index of an Annotation Header.
   # @param[out] output Always 2
   @staticmethod
   def headerIndex():
      return 2

   ##
   # @fn getHeader
   # @brief Extract header data from input bytes.
   # @details This function extracts header data from a buffer and stores it
   # within the object.  A version of the data buffer with the header bytes
   # chomped off is returned
   # @param[in] Raw unsigned bytes that contain the header at the beginning
   # @param[out] The input buffer with header bytes removed
   def getHeader(self,data):
      self.headerRecordLength = (data[1] << self.BYTE_SHIFT) | data[2]
      self.projectionName = "".join(map(chr,data[3:(3 + self.PROJECTION_NAME_LENGTH)])).strip()
      self.columnScalingFactor = 0
      for i in range(4):
         self.columnScalingFactor <<= self.BYTE_SHIFT
         self.columnScalingFactor |= data[35 + i]
      self.lineScalingFactor = 0
      for i in range(4):
         self.lineScalingFactor <<= self.BYTE_SHIFT
         self.lineScalingFactor |= data[39 + i]
      self.columnOffset = 0
      for i in range(4):
         self.columnOffset <<= self.BYTE_SHIFT
         self.columnOffset |= data[43 + i]
      self.lineOffset = 0
      for i in range(4):
         self.lineOffset <<= self.BYTE_SHIFT
         self.lineOffset |= data[47 + i]
      return data[self.headerRecordLength:len(data)]

   ##
   # @fn __str__
   # @brief Returns formatted header information as a string
   # @param[out] str Header information as a string
   def __str__(self):
      output = "Image Navigation Header\n"
      output += "Header Record Length: {0}\n".format(self.headerRecordLength)
      output += "Projection Name: {0}\n".format(self.projectionName)
      output += "Column Scaling Factor: {0}\n".format(self.columnScalingFactor)
      output += "Line Scaling Factor: {0}\n".format(self.lineScalingFactor)
      output += "Column Offset: {0}\n".format(self.columnOffset)
      output += "Line Offset: {0}\n".format(self.lineOffset)
      return output
