from AbstractHeader import AbstractHeader

##
# @brief Image Segment Identifier header class.
# @details Partition information for full-earth disc images that are
# partitioned
# @author Miguel Dominguez
# @date 2016
class ImageSegmentIdentificationHeader(AbstractHeader):

   ##
   # @fn headerIndex
   # @brief Return the header index of an Annotation Header.
   # @param[out] output Always 128
   @staticmethod
   def headerIndex():
      return 128

   ##
   # @fn getHeader
   # @brief Extract header data from input bytes.
   # @details This function extracts header data from a buffer and stores it
   # within the object.  A version of the data buffer with the header bytes
   # chomped off is returned
   # @param[in] Raw unsigned bytes that contain the header at the beginning
   # @param[out] The input buffer with header bytes removed
   def getHeader(self,data):
      self.headerRecordLength = (data[1] << self.BYTE_SHIFT) | data[2]
      self.imageId = (data[3] << self.BYTE_SHIFT) | data[4]
      self.segmentSequenceNumber = (data[5] << self.BYTE_SHIFT) | data[6]
      self.startColumn = (data[7] << self.BYTE_SHIFT) | data[8]
      self.startLine = (data[9] << self.BYTE_SHIFT) | data[10]
      self.maxSegment = (data[11] << self.BYTE_SHIFT) | data[12]
      self.maxColumn = (data[13] << self.BYTE_SHIFT) | data[14]
      self.maxRow = (data[15] << self.BYTE_SHIFT) | data[16]
      return data[self.headerRecordLength:len(data)]

   ##
   # @fn __str__
   # @brief Returns formatted header information as a string
   # @param[out] str Header information as a string
   def __str__(self):
      output = "Image Segment ID Header\n"
      output += "Header Record Length: {0}\n".format(self.headerRecordLength)
      output += "Image ID: {0}\n".format(self.imageId)
      output += "Segment Sequence #: {0}\n".format(self.segmentSequenceNumber)
      output += "Start Column: {0}\n".format(self.startColumn)
      output += "Start Line: {0}\n".format(self.startLine)
      output += "Max Segment: {0}\n".format(self.maxSegment)
      output += "Max Column: {0}\n".format(self.maxColumn)
      output += "Max Row: {0}\n".format(self.maxRow)
      return output
