#!/usr/bin/env python
# -*- coding: utf-8 -*-
# 
# Copyright 2016 The Aerospace Corporation.
# 
# This is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3, or (at your option)
# any later version.
# 
# This software is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this software; see the file COPYING.  If not, write to
# the Free Software Foundation, Inc., 51 Franklin Street,
# Boston, MA 02110-1301, USA.
# 

try:
    # try Enthought Canopy package structure first
    from enthought.traits.trait_base import ETSConfig
    ETSConfig.toolkit = 'qt4'
    import enthought.traits.api as tr
    import enthought.traits.ui.api as trui
except ImportError:
    # try generic package structure
    from traits.trait_base import ETSConfig
    ETSConfig.toolkit = 'qt4'
    import traits.api as tr
    import traitsui.api as trui

import logging 

# inheriting from StreamHandler gives a non-null handler method
class TraitsHandler(logging.Handler):
    def __init__(self, hastraits, trait, noformat=False, **kwds):
        assert trait in hastraits.trait_names()
        super(TraitsHandler, self).__init__(**kwds)
        self.hastraits = hastraits
        self.traitname = trait
        self.noformat = noformat
    
    # emit as an event
    def emit(self, rec):
        print self.formatter
        # create a custom trait to convert LogRecords to strings depending on trait?
        self.hastraits.set(**{self.traitname: rec if self.noformat else self.format(rec)})

from traitsui.tabular_adapter import TabularAdapter
class LogStringAdapter(TabularAdapter):
    # tuples: (column title, column trait name)
    columns = [('Log String', 'logstring')]
    logstring_text = tr.Property    # returns the text of trait logstring
    def _get_logstring_text(self): return self.item

class LogRecordAdapter(TabularAdapter):
    string_output = tr.true
    columns = tr.List(tr.Tuple)
    
class TraitsLogger(tr.HasTraits):
    emit = tr.Event
    logginglevels = tr.ListStr([k for k, v in sorted(logging._levelNames.iteritems(), key=lambda x: x[1]) 
                                if isinstance(k, str)])
    loglevel = tr.Enum(values='logginglevels')
    # uses strings for now
    loglist = tr.ListStr(editor=trui.TabularEditor(
                                editable=False,
                                show_titles=False,
                                adapter=LogStringAdapter()))
    logger = tr.Instance(logging.Logger)
    
    traits_view = trui.View(trui.VGroup(
                                trui.Group(trui.Item('loglevel', label='Log Level')),
                                trui.Group(trui.UReadonly('loglist'))
                            ),
                            width=500, resizable=True)
                            
    def _default_loglevel(self): return logging.getLevelName(self.logger.getEffectiveLevel())
    def _loglevel_changed(self, new): self.logger.setLevel(new)
    def _emit_fired(self, new): self.loglist.append(new)

import os
import os.path as osp

logging_format = '%(asctime)-15s | %(levelname)-8s | %(filename)16s:%(lineno)-4d | %(message)s'
logging.basicConfig(format=logging_format)   # default StreamHandler
logger = logging.getLogger(osp.splitext(osp.basename(__file__))[0])
logger.setLevel(logging.DEBUG)

import threading as th
from random import choice

def main(*args):
    log = TraitsLogger(logger=logger)       
    thandler = TraitsHandler(log, 'emit')
    thandler.setFormatter(logging.Formatter(logging_format))
    logger.addHandler(thandler)
    running = th.Event()
    t = th.Thread(target=makelogs, args=(running,log))
    t.start()
    running.set()
    log.configure_traits()
    running.clear()
    t.join()
    
def makelogs(running, log):
    import time
    running.wait()
    n = 1
    while running.is_set():
        time.sleep(1)
        logger.info('And the next choice is %s' % choice(['this', 'that', 'the other']))


import sys
if __name__ == '__main__':
    sys.exit(main(*sys.argv[1:]))
    
